@@ -1,7 +1,7 @@
 /* (c) 2002-2004 by Marcin Wiacek, Michal Cihar and others */
 /* based on some work from MyGnokii (www.mwiacek.com) */
 /* based on some work from Gnokii (www.gnokii.org)
- * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot 
+ * (C) 1999-2000 Hugh Blemings & Pavel Janik ml. (C) 2001-2004 Pawel Kot
  * GNU GPL version 2 or later
  */
 /* Due to a problem in the source code management, the names of some of
@@ -81,7 +81,7 @@ char *EncodeSpecialChars(unsigned char *buffer)
 			Buf[Pos2++] = '\\';
 			Buf[Pos2++] = 'n';
 			break;
-		case 13: 
+		case 13:
 			Buf[Pos2++] = '\\';
 			Buf[Pos2++] = 'r';
 			break;
@@ -178,9 +178,9 @@ char *mystrcasestr(unsigned const char *a, unsigned const char *b)
 	return strstr(A,B);
 }
 
-unsigned int UnicodeLength(const unsigned char *str)
+size_t UnicodeLength(const unsigned char *str)
 {
-	unsigned int len = 0;
+	size_t len = 0;
 
 	if (str == NULL) return 0;
 
@@ -190,9 +190,9 @@ unsigned int UnicodeLength(const unsigned char *str)
 }
 
 /* Convert Unicode char saved in src to dest */
-unsigned int EncodeWithUnicodeAlphabet(const unsigned char *src, wchar_t *dest)
+int EncodeWithUnicodeAlphabet(const unsigned char *src, wchar_t *dest)
 {
-	char retval;
+	int retval;
 
         switch (retval = mbtowc(dest, src, MB_CUR_MAX)) {
                 case -1 :
@@ -202,10 +202,10 @@ unsigned int EncodeWithUnicodeAlphabet(const unsigned char *src, wchar_t *dest)
 }
 
 /* Convert Unicode char saved in src to dest */
-unsigned int DecodeWithUnicodeAlphabet(wchar_t src, unsigned char *dest)
+int DecodeWithUnicodeAlphabet(wchar_t src, unsigned char *dest)
 {
         int retval;
-        
+
         switch (retval = wctomb(dest, src)) {
                 case -1:
                         *dest = '?';
@@ -266,11 +266,11 @@ unsigned char *DecodeUnicodeConsole(const unsigned char *src)
 }
 
 /* Encode string to Unicode. Len is number of input chars */
-void EncodeUnicode (unsigned char *dest, const unsigned char *src, int len)
+void EncodeUnicode (unsigned char *dest, const char *src, int len)
 {
 	int 		i_len = 0, o_len;
  	wchar_t 	wc;
- 
+
 	for (o_len = 0; i_len < len; o_len++) {
 		i_len += EncodeWithUnicodeAlphabet(&src[i_len], &wc);
 		dest[o_len*2]		= (wc >> 8) & 0xff;
@@ -470,7 +470,7 @@ void DecodeDefault (unsigned char *dest, const unsigned char *src, int len, bool
 	bool	FoundSpecial = false;
 
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, src, len);
+	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, src, len);
 #endif
 
 	for (i = 0; i < len; i++) {
@@ -478,7 +478,7 @@ void DecodeDefault (unsigned char *dest, const unsigned char *src, int len, bool
 		if ((i < (len-1)) && UseExtensions) {
 			j=0;
 			while (GSM_DefaultAlphabetCharsExtension[j][0]!=0x00) {
-				if (GSM_DefaultAlphabetCharsExtension[j][0]==src[i] && 
+				if (GSM_DefaultAlphabetCharsExtension[j][0]==src[i] &&
 				    GSM_DefaultAlphabetCharsExtension[j][1]==src[i+1]) {
 					FoundSpecial = true;
 					dest[current++] = GSM_DefaultAlphabetCharsExtension[j][2];
@@ -509,7 +509,7 @@ void DecodeDefault (unsigned char *dest, const unsigned char *src, int len, bool
 	dest[current++]=0;
 	dest[current++]=0;
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, dest, UnicodeLength(dest)*2);
+	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, dest, UnicodeLength(dest)*2);
 #endif
 }
 
@@ -535,7 +535,7 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, int *len, bool
 	bool	FoundSpecial,FoundNormal;
 
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, src, (*len)*2);
+	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, src, (*len)*2);
 #endif
 
 	for (i = 0; i < *len; i++) {
@@ -603,7 +603,7 @@ void EncodeDefault(unsigned char *dest, const unsigned char *src, int *len, bool
 	}
 	dest[current]=0;
 #ifdef DEBUG
-	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(di.df, di.dl, dest, current);
+	if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) DumpMessage(&di, dest, current);
 #endif
 
 	*len = current;
@@ -745,7 +745,7 @@ void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, boo
 		length=length / 2 + 1;
 	}
 
-	/*without leading byte with format of number*/  
+	/*without leading byte with format of number*/
 	length--;
 
 	switch ((Number[1] & 112)) {
@@ -766,6 +766,7 @@ void GSM_UnpackSemiOctetNumber(unsigned char *retval, unsigned char *Number, boo
 		break;
 	}
 
+	dbgprintf("Len %i\n",length);
 	EncodeUnicode(retval,Buffer,strlen(Buffer));
 }
 
@@ -797,7 +798,7 @@ int GSM_PackSemiOctetNumber(unsigned char *Number, unsigned char *Output, bool s
 	for (i=0;i<length;i++) {
 		/* first byte is '+'. Number can be international */
 		if (i==0 && buffer[i]=='+') {
-			format=NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;  
+			format=NUMBER_INTERNATIONAL_NUMBERING_PLAN_ISDN;
 		} else {
 			/*char is not number. It must be alphanumeric*/
 			if (!isdigit(buffer[i])) format=NUMBER_ALPHANUMERIC_NUMBERING_PLAN_UNKNOWN;
@@ -983,7 +984,7 @@ void GetBufferInt(unsigned char *Source,
 		z=z/2;
 		i++;
 	}
-	*integer=l;  
+	*integer=l;
 	(*CurrentBit) = (*CurrentBit) + i;
 }
 
@@ -993,7 +994,7 @@ void GetBufferI(unsigned char 	*Source,
 		int 		BitsToProcess)
 {
 	int l=0,z,i=0;
-	
+
 	z = 1<<(BitsToProcess-1);
 
 	while (i!=BitsToProcess) {
@@ -1001,7 +1002,7 @@ void GetBufferI(unsigned char 	*Source,
 		z=z>>1;
 		i++;
 	}
-	*result=l;  
+	*result=l;
 	(*CurrentBit) = (*CurrentBit) + i;
 }
 
@@ -1021,7 +1022,7 @@ void EncodeUnicodeSpecialNOKIAChars(unsigned char *dest, const unsigned char *sr
 				dest[current++]	= '~';
 			} else {
 				dest[current++]	= 0x00;
-				dest[current++]	= 0x01;					
+				dest[current++]	= 0x01;
 				dest[current++]	= src[i*2];
 				dest[current++]	= src[i*2+1];
 			}
@@ -1082,7 +1083,7 @@ bool mystrncasecmp(unsigned const char *a, unsigned const char *b, int num)
 	if (a == NULL || b == NULL) return false;
 
 	if (num == 0) num = -1;
-   
+
 	for (i = 0; i != num; i++) {
 		if (a[i] == 0x00 && b[i] == 0x00) return true;
 		if (a[i] == 0x00 || b[i] == 0x00) return false;
@@ -1102,7 +1103,7 @@ bool mywstrncasecmp(unsigned const  char *a, unsigned const  char *b, int num)
         if (a == NULL || b == NULL) return false;
 
 	if (num == 0) num = -1;
-	
+
 	for (i = 0; i != num; i++) {
 		if ((a[i*2] == 0x00 && a[i*2+1] == 0x00) && (b[i*2] == 0x00 && b[i*2+1] == 0x00)) return true;
 		if ((a[i*2] == 0x00 && a[i*2+1] == 0x00) || (b[i*2] == 0x00 && b[i*2+1] == 0x00)) return false;
@@ -1117,7 +1118,7 @@ bool mywstrncasecmp(unsigned const  char *a, unsigned const  char *b, int num)
 bool mywstrncmp(unsigned const  char *a, unsigned const  char *b, int num)
 {
 	int i=0;
-  
+
 	while (1) {
 		if (a[i*2] != b[i*2] || a[i*2+1] != b[i*2+1]) return false;
 		if (a[i*2] == 0x00 && a[i*2+1] == 0x00) return true;
@@ -1145,7 +1146,8 @@ bool myiswspace(unsigned const char *src)
 	}
 	return false;
 #else
-	return iswspace(wc);
+	if (iswspace(wc)) return true;
+	return false;
 #endif
 }
 
@@ -1294,45 +1296,43 @@ void StringToDouble(char *text, double *d)
 }
 
 /* When char can be converted, convert it from Unicode to UTF8 */
-bool EncodeWithUTF8Alphabet(unsigned char mychar1, unsigned char mychar2, unsigned char *ret1, unsigned char *ret2)
+int EncodeWithUTF8Alphabet2(unsigned char mychar1, unsigned char mychar2, unsigned char *ret)
 {
-	unsigned char	mychar3,mychar4;
-	int		j=0;
-      
-	if (mychar1>0x00 || mychar2>128) {
-		mychar3=0x00;
-		mychar4=128;
-		while (true) {
-			if (mychar3==mychar1) {
-				if (mychar4+64>=mychar2) {
-					*ret1=j+0xc2;
-					*ret2=0x80+(mychar2-mychar4);
-					return true;
-				}
-			}
-			if (mychar4==192) {
-				mychar3++;
-				mychar4=0;
-			} else {
-				mychar4=mychar4+64;
-			}
-			j++;
-		}
+	int src = mychar1*256+mychar2;
+
+	//Unicode 0080-07FF -> UTF8 110xxxxx 10xxxxxx
+	if (src >=128 && src <=2047) {
+		ret[0] = 192 + (src / 64);
+		ret[1] = 128 + (src % 64);
+		return 2;
 	}
-	return false;
+	//Unicode 0800-FFFF -> UTF8 1110xxxx 10xxxxxx 10xxxxxx
+	if (src >2047) {
+		ret[0] = 224 + (src / 4096);
+		ret[1] = 128 + ((src / 64) % 64);
+		ret[2] = 128 + (src % 64);
+		return 3;
+	}
+
+	//Unicode 0000-007F -> UTF8 0xxxxxxx
+	ret[0] = mychar2;
+	return 1;
 }
 
 /* Make UTF8 string from Unicode input string */
 bool EncodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src)
 {
-	int		i,j=0;
-	unsigned char	mychar1, mychar2;
+	int		i,j=0,z,w;
+	unsigned char	mychar[3];
 	bool		retval = false;
-	
+
 	for (i = 0; i < (int)(UnicodeLength(src)); i++) {
-		if (EncodeWithUTF8Alphabet(src[i*2],src[i*2+1],&mychar1,&mychar2)) {
-			sprintf(dest+j, "=%02X=%02X",mychar1,mychar2);
-			j	= j+6;
+		z = EncodeWithUTF8Alphabet2(src[i*2],src[i*2+1],mychar);
+		if (z>1) {
+			for (w=0;w<z;w++) {
+				sprintf(dest+j, "=%02X",mychar[w]);
+				j = j+3;				
+			}
 			retval  = true;
 		} else {
 			j += DecodeWithUnicodeAlphabet(((wchar_t)(src[i*2]*256+src[i*2+1])), dest + j);
@@ -1344,17 +1344,18 @@ bool EncodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src)
 
 bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 {
-	int		i,j=0;
-	unsigned char	mychar1, mychar2;
+	int		i,j=0,z;
+	unsigned char	mychar[3];
 	bool		retval = false;
-	
+
 	for (i = 0; i < (int)(UnicodeLength(src)); i++) {
-		if (EncodeWithUTF8Alphabet(src[i*2],src[i*2+1],&mychar1,&mychar2)) {
-			sprintf(dest+j, "%c%c",mychar1,mychar2);
-			j	= j+2;
-			retval  = true;
-	    	} else {
-			j += DecodeWithUnicodeAlphabet(((wchar_t)(src[i*2]*256+src[i*2+1])), dest + j);
+		z = EncodeWithUTF8Alphabet2(src[i*2],src[i*2+1],mychar);
+		if (z>1) {
+			memcpy(dest+j,mychar,z);
+			j+=z;
+			retval = true;
+		} else {
+			j+= DecodeWithUnicodeAlphabet(((wchar_t)(src[i*2]*256+src[i*2+1])), dest + j);
 	    	}
 	}
 	dest[j++]=0;
@@ -1362,46 +1363,50 @@ bool EncodeUTF8(unsigned char *dest, const unsigned char *src)
 }
 
 /* Decode UTF8 char to Unicode char */
-wchar_t DecodeWithUTF8Alphabet(unsigned char mychar3, unsigned char mychar4)
+int DecodeWithUTF8Alphabet2(unsigned char *src, wchar_t *dest, int len)
 {
-	unsigned char	mychar1, mychar2;
-	int		j;
-	
-	mychar1=0x00;
-	mychar2=128;
-	for(j=0;j<mychar3-0xc2;j++) {
-		if (mychar2==192) {
-			mychar1++;
-			mychar2 = 0;
-		} else {
-			mychar2 = mychar2+64;
-		}
+	if (len < 1) return 0;
+	if (src[0] < 128) {
+		(*dest) = src[0];
+		return 1;
 	}
-	mychar2 = mychar2+(mychar4-0x80);
-	return mychar2 | (mychar1 << 8);
+	if (src[0] < 194) return 0;
+	if (src[0] < 224) {
+		if (len < 2) return 0;
+		(*dest) = (src[0]-192)*64 + (src[1]-128);
+		return 2;
+	}
+	if (src[0] < 240) {
+		if (len < 3) return 0;
+		(*dest) = (src[0]-224)*4096 + (src[1]-128)*64 + (src[2]-128);
+		return 3;
+	}
+	return 0;
 }
 
 /* Make Unicode string from UTF8 string */
 void DecodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src, int len)
 {
-	int 		i=0,j=0;
-	unsigned char	mychar1, mychar2;
+	int 		i=0,j=0,z;
+	unsigned char	mychar[10];
 	wchar_t		ret;
-	
+
 	while (i<=len) {
-		if (len-6>=i) {
-			/* Need to have correct chars */
-			if (src[i]  =='=' && DecodeWithHexBinAlphabet(src[i+1])!=-1
-	                                  && DecodeWithHexBinAlphabet(src[i+2])!=-1 &&
-			    src[i+3]=='=' && DecodeWithHexBinAlphabet(src[i+4])!=-1 &&
-	                                     DecodeWithHexBinAlphabet(src[i+5])!=-1) {
-				mychar1	= 16*DecodeWithHexBinAlphabet(src[i+1])+DecodeWithHexBinAlphabet(src[i+2]);
-				mychar2	= 16*DecodeWithHexBinAlphabet(src[i+4])+DecodeWithHexBinAlphabet(src[i+5]);
-				ret 	= DecodeWithUTF8Alphabet(mychar1,mychar2);
-				i 	= i+6;
-			} else {
-				i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
-			}   
+		z=0;
+		while (true) {
+			if (src[z*3+i] != '=' || z*3+i+3>len ||
+			    DecodeWithHexBinAlphabet(src[z*3+i+1])==-1 ||
+			    DecodeWithHexBinAlphabet(src[z*3+i+2])==-1) {
+				break;
+			}
+			mychar[z] = 16*DecodeWithHexBinAlphabet(src[z*3+i+1])+DecodeWithHexBinAlphabet(src[z*3+i+2]);
+			if (z==0 && mychar[0]<194) break;
+			z++;
+		}
+		if (z>0) {
+			i+=z*3;
+			// we ignore wrong sequence
+			if (DecodeWithUTF8Alphabet2(mychar,&ret,z)==0) continue;
 		} else {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
 		}
@@ -1414,19 +1419,15 @@ void DecodeUTF8QuotedPrintable(unsigned char *dest, const unsigned char *src, in
 
 void DecodeUTF8(unsigned char *dest, const unsigned char *src, int len)
 {
-	int 		i=0,j=0;
+	int 		i=0,j=0,z;
 	wchar_t		ret;
-	
+
 	while (i<=len) {
-		if (len-2>=i) {
-			if (src[i] >= 0xC2) {
-				ret 	= DecodeWithUTF8Alphabet(src[i],src[i+1]);
-				i 	= i+2;
-			} else {
-				i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
-			}
-		} else {
+		z = DecodeWithUTF8Alphabet2((unsigned char *)src+i,&ret,len-i);
+		if (z<2) {
 			i+=EncodeWithUnicodeAlphabet(&src[i], &ret);
+		} else {
+			i+=z;
 		}
 		dest[j++] = (ret >> 8) & 0xff;
 		dest[j++] = ret & 0xff;
@@ -1439,7 +1440,7 @@ void DecodeUTF7(unsigned char *dest, const unsigned char *src, int len)
 {
 	int 		i=0,j=0,z,p;
 	wchar_t		ret;
-	
+
 	while (i<=len) {
 		if (len-5>=i) {
 			if (src[i] == '+') {
@@ -1527,7 +1528,7 @@ void EncodeBASE64(const unsigned char *Input, unsigned char *Output, int Length)
 }
 
 static void DecodeBASE64Block(unsigned char in[4], unsigned char out[3])
-{   
+{
 	out[0] = (unsigned char) (in[0] << 2 | in[1] >> 4);
 	out[1] = (unsigned char) (in[1] << 4 | in[2] >> 2);
 	out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
