@@ -1,4 +1,4 @@
-/* (c) 2002-2004 by Marcin Wiacek and Michal Cihar */
+/* (c) 2002-2005 by Marcin Wiacek and Michal Cihar */
 
 #ifndef atgen_h
 #define atgen_h
@@ -47,11 +47,12 @@ typedef enum {
 } GSM_AT_Manufacturer;
 
 typedef enum {
-	AT_PBK_HEX = 1,
-	AT_PBK_GSM,
-	AT_PBK_UCS2,
-	AT_PBK_PCCP437
-} GSM_AT_PBK_Charset;
+	AT_CHARSET_HEX = 1,
+	AT_CHARSET_GSM,
+	AT_CHARSET_UCS2,
+	AT_CHARSET_UTF8,
+	AT_CHARSET_PCCP437
+} GSM_AT_Charset;
 
 typedef enum {
 	AT_AVAILABLE = 1,
@@ -84,9 +85,10 @@ typedef struct {
 	char			PBKMemories[AT_PBK_MAX_MEMORIES + 1]; /* Supported by phone PBK memories 		*/
 	int			NextMemoryEntry;  	/* Next empty memory entry					*/
 	int			FirstMemoryEntry;	/* First memory entry to be read				*/
-	GSM_AT_PBK_Charset	PBKCharset;	  	/* Last read PBK charset					*/
-	bool			UCS2CharsetFailed;	/* Whether setting of UCS2 charset has already failed 		*/
-	bool			NonUCS2CharsetFailed;	/* Whether setting of non-UCS2 charset has already failed 	*/
+	GSM_AT_Charset		Charset;	  	/* Charset set in phone						*/
+	bool			EncodedCommands;	/* True if phone encodes also commands and not only values	*/
+	GSM_AT_Charset		UnicodeCharset;	  	/* Charset to use for unicode mode				*/
+	GSM_AT_Charset		NormalCharset;	  	/* Charset to use for non unicode mode				*/
 	GSM_AT_SBNR		PBKSBNR;
 	int			NumberLength;
 	int			TextLength;
@@ -101,6 +103,25 @@ typedef struct {
 	GSM_MemoryType		SMSMemory;	  	/* Last read SMS memory 					*/
 	GSM_AT_SMS_Modes	SMSMode;	  	/* PDU or TEXT mode for SMS ? 					*/
 
+	/**
+	 * Mode used for incoming message notification.
+	 */
+	int			CNMIMode;
+	/**
+	 * Procedure used for incoming message notification.
+	 */
+	int			CNMIProcedure;
+	/**
+	 * Procedure used for incoming delivery report message notification.
+	 */
+	int			CNMIDeliverProcedure;
+#ifdef GSM_ENABLE_CELLBROADCAST
+	/**
+	 * Mode used for incoming broadcast message notification.
+	 */
+	int			CNMIBroadcastProcedure;
+#endif
+
 	bool			OBEX;
 	GSM_File		file;
 } GSM_Phone_ATGENData;
