@@ -1,4 +1,4 @@
-/* (c) 2002-2004 by Marcin Wiacek */
+/* (c) 2002-2006 by Marcin Wiacek */
 
 #include <ctype.h>
 #include <string.h>
@@ -39,14 +39,14 @@ void GSM_Find_Free_Used_SMS2(GSM_Coding_Type Coding,GSM_SMSMessage SMS, int *Use
 	int UsedBytes;
 
 	switch (Coding) {
-	case SMS_Coding_Default:
+	case SMS_Coding_Default_No_Compression:
 		FindDefaultAlphabetLen(SMS.Text,&UsedBytes,UsedText,500);
 		UsedBytes = *UsedText * 7 / 8;
 		if (UsedBytes * 8 / 7 != *UsedText) UsedBytes++;
 		*FreeBytes = GSM_MAX_8BIT_SMS_LENGTH - SMS.UDH.Length - UsedBytes;
 		*FreeText = (GSM_MAX_8BIT_SMS_LENGTH - SMS.UDH.Length) * 8 / 7 - *UsedText;
 		break;
-	case SMS_Coding_Unicode:
+	case SMS_Coding_Unicode_No_Compression:
 		*UsedText = UnicodeLength(SMS.Text);
 		UsedBytes = *UsedText * 2;
 		*FreeBytes = GSM_MAX_8BIT_SMS_LENGTH - SMS.UDH.Length - UsedBytes;
@@ -57,6 +57,8 @@ void GSM_Find_Free_Used_SMS2(GSM_Coding_Type Coding,GSM_SMSMessage SMS, int *Use
 		*FreeBytes = GSM_MAX_8BIT_SMS_LENGTH - SMS.UDH.Length - UsedBytes;
 		*FreeText = *FreeBytes;
 		break;
+	default:
+		break;
 	}
 	dbgprintf("UDH len %i, UsedBytes %i, FreeText %i, UsedText %i, FreeBytes %i\n",SMS.UDH.Length,UsedBytes,*FreeText,*UsedText,*FreeBytes);
 }
@@ -105,7 +107,7 @@ GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 		dbgprintf("copy %i\n",Copy);
 
 		switch (Coding) {
-		case SMS_Coding_Default:
+		case SMS_Coding_Default_No_Compression:
 			FindDefaultAlphabetLen(Buffer,&i,&j,FreeText);
 			dbgprintf("def length %i %i\n",i,j);
 			SMS->SMS[SMS->Number].Text[UnicodeLength(SMS->SMS[SMS->Number].Text)*2+i*2]   = 0;
@@ -115,7 +117,7 @@ GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 			*CopiedSMSText 	= j;
 			SMS->SMS[SMS->Number].Length += i;
 			break;
-		case SMS_Coding_Unicode:
+		case SMS_Coding_Unicode_No_Compression:
 			SMS->SMS[SMS->Number].Text[UnicodeLength(SMS->SMS[SMS->Number].Text)*2+Copy*2]   = 0;
 			SMS->SMS[SMS->Number].Text[UnicodeLength(SMS->SMS[SMS->Number].Text)*2+Copy*2+1] = 0;
 			memcpy(SMS->SMS[SMS->Number].Text+UnicodeLength(SMS->SMS[SMS->Number].Text)*2,Buffer,Copy*2);
@@ -127,6 +129,8 @@ GSM_Error GSM_AddSMS_Text_UDH(GSM_MultiSMSMessage 	*SMS,
 			SMS->SMS[SMS->Number].Length += Copy;
 			*CopiedText = *CopiedSMSText = Copy;
 			break;
+		default:
+			break;
 		}
 		dbgprintf("Text added\n");
 	}
@@ -173,7 +177,7 @@ void GSM_MakeMultiPartSMS(GSM_MultiSMSMessage	*SMS,
 	SMS->Number++;
 
 	UDHID = GSM_MakeSMSIDFromTime();
-	GSM_GetCurrentDateTime (&Date);	
+	GSM_GetCurrentDateTime (&Date);
 	for (j=0;j<SMS->Number;j++) {
 		SMS->SMS[j].UDH.Type 		= UDHType;
 		SMS->SMS[j].UDH.ID8bit 		= UDHID;
@@ -211,7 +215,7 @@ static void GSM_EncodeSMS30MultiPartSMS(GSM_MultiPartSMSInfo *Info,
 	/*SM version. Here 3.0*/
 	Buffer[(*Length)++] = 0x30;
 
-	if (Info->Entries[0].ID == SMS_NokiaProfileLong) {	
+	if (Info->Entries[0].ID == SMS_NokiaProfileLong) {
 		if (Info->Entries[0].Buffer != NULL) {
 			if (Info->Entries[0].Buffer[0]!=0x00 || Info->Entries[0].Buffer[1]!=0x00) {
 				Buffer[(*Length)++] = SM30_PROFILENAME;
@@ -311,7 +315,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 		p += i;
 		SMS->SMS[SMS->Number].Length = i;
 		SMS->Number++;
-		
+
 	}
 
 	/* Linked sms UDH */
@@ -323,7 +327,7 @@ GSM_Error GSM_EncodeAlcatelMultiPartSMS(GSM_MultiSMSMessage 	*SMS,
 			SMS->SMS[i].UDH.Text[SMS->SMS[i].UDH.Length-1] = i+1;
 		}
 	}
-        
+
         return ERR_NONE;
 }
 
@@ -408,7 +412,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			break;
 		case SMS_ConcatenatedTextLong:
 		case SMS_ConcatenatedTextLong16bit:
-		
+
 			/* This covers situation, when somebody will call function
 			 * with two or more SMS_Concatenated.... entries only.
 			 * It will be still only linked sms, but functions below
@@ -419,7 +423,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			 */
 			textnum ++;
 			if (textnum > 1) EMS = true;
-			
+
 			if (Info->Entries[i].Left   || Info->Entries[i].Right      ||
 			    Info->Entries[i].Center || Info->Entries[i].Large      ||
 			    Info->Entries[i].Small  || Info->Entries[i].Bold       ||
@@ -565,7 +569,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			/* FIXME: It wasn't checked */
 			UDH = UDH_NokiaPhonebookLong;
 		}
-		Coding = SMS_Coding_Default;
+		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
@@ -580,21 +584,21 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 			UDH = UDH_NokiaPhonebookLong;
 			/* Here can be also 8 bit coding */
 		}
-		Coding = SMS_Coding_Default;
+		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_VCARD10Long:
 		GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard10);
 		if (Length>GSM_MAX_SMS_LENGTH) UDH = UDH_ConcatenatedMessages;
-		Coding = SMS_Coding_Default;
+		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
 	case SMS_VCARD21Long:
 		GSM_EncodeVCARD(Buffer,&Length,Info->Entries[0].Phonebook,true,Nokia_VCard21);
 		if (Length>GSM_MAX_SMS_LENGTH) UDH = UDH_ConcatenatedMessages;
-		Coding = SMS_Coding_Default;
+		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
@@ -603,14 +607,14 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		if (error != ERR_NONE) return error;
 		/* Is 1 SMS ? 8 = length of ..SCKE4 */
 		if (Length<=GSM_MAX_SMS_LENGTH-8) {
-			sprintf(Buffer,"//SCKE4 ");  
+			sprintf(Buffer,"//SCKE4 ");
 			Length = 8;
 			GSM_EncodeVCALENDAR(Buffer,&Length,Info->Entries[0].Calendar,true,Nokia_VCalendar);
 		} else {
 			UDH = UDH_NokiaCalendarLong;
 			/* can be here 8 bit coding ? */
 		}
-		Coding = SMS_Coding_Default;
+		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
@@ -618,7 +622,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		error=GSM_EncodeVTODO(Buffer,&Length,Info->Entries[0].ToDo,true,Nokia_VToDo);
 		if (error != ERR_NONE) return error;
 		UDH = UDH_NokiaCalendarLong;
-		Coding = SMS_Coding_Default;
+		Coding = SMS_Coding_Default_No_Compression;
 		memcpy(Buffer2,Buffer,Length);
 		EncodeUnicode(Buffer,Buffer2,Length);
 		break;
@@ -646,11 +650,11 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		GSM_EncodeUDHHeader(&UDHHeader);
 		memcpy(Buffer,Info->Entries[0].Buffer,UnicodeLength(Info->Entries[0].Buffer)*2+2);
 		if (Info->UnicodeCoding) {
-			Coding = SMS_Coding_Unicode;
+			Coding = SMS_Coding_Unicode_No_Compression;
 			Length = UnicodeLength(Info->Entries[0].Buffer);
 			if (Length>(140-UDHHeader.Length)/2) Length = (140-UDHHeader.Length)/2;
 		} else {
-			Coding = SMS_Coding_Default;
+			Coding = SMS_Coding_Default_No_Compression;
 			FindDefaultAlphabetLen(Info->Entries[0].Buffer,&Length,&smslen,(GSM_MAX_8BIT_SMS_LENGTH-UDHHeader.Length)*8/7);
 		}
 		break;
@@ -663,9 +667,9 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 #ifdef DEBUG
 		if (di.dl == DL_TEXTALL || di.dl == DL_TEXTALLDATE) {
 			dbgprintf("Info->Entries[0].Buffer:\n");
-			DumpMessage(di.df, di.dl, Info->Entries[0].Buffer, UnicodeLength(Info->Entries[0].Buffer)*2);
+			DumpMessage(&di, Info->Entries[0].Buffer, UnicodeLength(Info->Entries[0].Buffer)*2);
 			dbgprintf("Buffer:\n");
-			DumpMessage(di.df, di.dl, Buffer, UnicodeLength(Buffer)*2);
+			DumpMessage(&di, Buffer, UnicodeLength(Buffer)*2);
 		}
 #endif
 		Info->UnicodeCoding = false;
@@ -683,7 +687,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 		memcpy(Buffer,Info->Entries[0].Buffer,UnicodeLength(Info->Entries[0].Buffer)*2+2);
 		UDH = UDH_NoUDH;
 		if (Info->UnicodeCoding) {
-			Coding = SMS_Coding_Unicode;
+			Coding = SMS_Coding_Unicode_No_Compression;
 			Length = UnicodeLength(Info->Entries[0].Buffer);
 			if (Info->Entries[0].ID == SMS_ConcatenatedTextLong16bit ||
 			    Info->Entries[0].ID == SMS_ConcatenatedAutoTextLong16bit) {
@@ -692,7 +696,7 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo		*Info,
 				if (Length>70) UDH=UDH_ConcatenatedMessages;
 			}
 		} else {
-			Coding = SMS_Coding_Default;
+			Coding = SMS_Coding_Default_No_Compression;
 			FindDefaultAlphabetLen(Info->Entries[0].Buffer,&Length,&smslen,5000);
 			if (Info->Entries[0].ID == SMS_ConcatenatedTextLong16bit ||
 			    Info->Entries[0].ID == SMS_ConcatenatedAutoTextLong16bit) {
@@ -717,6 +721,7 @@ void GSM_ClearMultiPartSMSInfo(GSM_MultiPartSMSInfo *Info)
 		Info->Entries[i].Ringtone	= NULL;
 		Info->Entries[i].Bitmap		= NULL;
 		Info->Entries[i].Bookmark	= NULL;
+		Info->Entries[i].File		= NULL;
 		Info->Entries[i].Settings	= NULL;
 		Info->Entries[i].MMSIndicator	= NULL;
 		Info->Entries[i].Phonebook	= NULL;
@@ -734,7 +739,7 @@ void GSM_ClearMultiPartSMSInfo(GSM_MultiPartSMSInfo *Info)
 		Info->Entries[i].Italic		= false;
 		Info->Entries[i].Underlined	= false;
 		Info->Entries[i].Strikethrough	= false;
-		
+
 		Info->Entries[i].RingtoneNotes	= 0;
 	}
 	Info->Unknown		= false;
@@ -776,9 +781,11 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 			    GSM_MultiSMSMessage		*SMS,
 			    bool			ems)
 {
-	int 	i, Length = 0;
-	char	Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
-	bool 	emsexist = false;
+	int 			i, Length = 0;
+	unsigned int	j;
+	char			Buffer[GSM_MAX_SMS_LENGTH*2*MAX_MULTI_SMS];
+	bool 			emsexist = false;
+	GSM_SiemensOTASMSInfo	SiemensInfo;
 
 	GSM_ClearMultiPartSMSInfo(Info);
 	if (ems) {
@@ -790,12 +797,35 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 				emsexist = false;
 				break;
 			}
-		}	
+		}
 	}
 
 	/* EMS decoding */
 	if (emsexist) return GSM_DecodeEMSMultiPartSMS(Info,SMS);
 
+	/* Siemens OTA */
+	if (GSM_DecodeSiemensOTASMS(&SiemensInfo,&SMS->SMS[0])) {
+		Info->Entries[0].File = (GSM_File *)malloc(sizeof(GSM_File));
+		if (Info->Entries[0].File == NULL) return false;
+		Info->Entries[0].File->Buffer 	= NULL;
+		Info->Entries[0].File->Used 	= 0;
+		for (i=0;i<SMS->Number;i++) {
+			GSM_DecodeSiemensOTASMS(&SiemensInfo,&SMS->SMS[i]);
+			j = SiemensInfo.AllDataLen - Info->Entries[0].File->Used;
+			if (j>SiemensInfo.DataLen) j = SiemensInfo.DataLen;
+			Info->Entries[0].File->Buffer = realloc(Info->Entries[0].File->Buffer,j+Info->Entries[0].File->Used);
+			memcpy(Info->Entries[0].File->Buffer+Info->Entries[0].File->Used,SiemensInfo.Data,j);
+			Info->Entries[0].File->Used += j;
+		}
+		if (SiemensInfo.AllDataLen == Info->Entries[0].File->Used) {
+			Info->Entries[0].ID 	= SMS_SiemensFile;
+			Info->EntriesNum	= 1;
+			EncodeUnicode(Info->Entries[0].File->Name,SiemensInfo.DataName,strlen(SiemensInfo.DataName));
+			return true;
+		}
+		free(Info->Entries[0].File->Buffer);
+	}
+
 	/* Smart Messaging decoding */
 	if (SMS->SMS[0].UDH.Type == UDH_NokiaRingtone && SMS->Number == 1) {
 		Info->Entries[0].Ringtone = (GSM_Ringtone *)malloc(sizeof(GSM_Ringtone));
@@ -853,14 +883,9 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 			switch (Buffer[i]) {
 			case SM30_ISOTEXT:
 				dbgprintf("ISO 8859-2 text\n");
-				Info->Unknown = true;
 				break;
 			case SM30_UNICODETEXT:
 				dbgprintf("Unicode text\n");
-				memcpy(Info->Entries[0].Bitmap->Bitmap[0].Text,Buffer+i+3,Buffer[i+1]*256+Buffer[i+2]);
-				Info->Entries[0].Bitmap->Bitmap[0].Text[Buffer[i+1]*256 + Buffer[i+2]] 	= 0;
-				Info->Entries[0].Bitmap->Bitmap[0].Text[Buffer[i+1]*256 + Buffer[i+2]+ 1] 	= 0;
-				dbgprintf("Unicode Text \"%s\"\n",DecodeUnicodeString(Info->Entries[0].Bitmap->Bitmap[0].Text));
 				break;
 			case SM30_OTA:
 				dbgprintf("OTA bitmap as Picture Image\n");
@@ -890,6 +915,37 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 			i = i + Buffer[i+1]*256 + Buffer[i+2] + 3;
 			dbgprintf("%i %i\n",i,Length);
 		}
+		i=1;
+		while (i!=Length) {
+			switch (Buffer[i]) {
+			case SM30_ISOTEXT:
+				dbgprintf("ISO 8859-2 text\n");
+				EncodeUnicode (Info->Entries[0].Bitmap->Bitmap[0].Text, Buffer+i+3, Buffer[i+2]);
+				dbgprintf("ISO Text \"%s\"\n",DecodeUnicodeString(Info->Entries[0].Bitmap->Bitmap[0].Text));
+				break;
+			case SM30_UNICODETEXT:
+				dbgprintf("Unicode text\n");
+				memcpy(Info->Entries[0].Bitmap->Bitmap[0].Text,Buffer+i+3,Buffer[i+1]*256+Buffer[i+2]);
+				Info->Entries[0].Bitmap->Bitmap[0].Text[Buffer[i+1]*256 + Buffer[i+2]] 	= 0;
+				Info->Entries[0].Bitmap->Bitmap[0].Text[Buffer[i+1]*256 + Buffer[i+2]+ 1] 	= 0;
+				dbgprintf("Unicode Text \"%s\"\n",DecodeUnicodeString(Info->Entries[0].Bitmap->Bitmap[0].Text));
+				break;
+			case SM30_OTA:
+				dbgprintf("OTA bitmap as Picture Image\n");
+				break;
+			case SM30_RINGTONE:
+				dbgprintf("RTTL ringtone\n");
+				break;
+			case SM30_PROFILENAME:
+				dbgprintf("Profile Name\n");
+				break;
+			case SM30_SCREENSAVER:
+				dbgprintf("OTA bitmap as Screen Saver\n");
+				break;
+			}
+			i = i + Buffer[i+1]*256 + Buffer[i+2] + 3;
+			dbgprintf("%i %i\n",i,Length);
+		}
 		return true;
 	}
 
@@ -907,24 +963,26 @@ bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo	*Info,
 			case SMS_Coding_8bit:
 				Info->Entries[0].Buffer = realloc(Info->Entries[0].Buffer, Length + SMS->SMS[i].Length + 2);
 				if (Info->Entries[0].Buffer == NULL) return false;
-				
+
 				memcpy(Info->Entries[0].Buffer + Length, SMS->SMS[i].Text, SMS->SMS[i].Length);
 				Length=Length+SMS->SMS[i].Length;
 				break;
-			case SMS_Coding_Unicode:
+			case SMS_Coding_Unicode_No_Compression:
 				if (Info->Entries[0].ID == SMS_ConcatenatedTextLong) {
 					Info->Entries[0].ID = SMS_ConcatenatedAutoTextLong;
 				}
 				if (Info->Entries[0].ID == SMS_ConcatenatedTextLong16bit) {
 					Info->Entries[0].ID = SMS_ConcatenatedAutoTextLong16bit;
 				}
-			case SMS_Coding_Default:
+			case SMS_Coding_Default_No_Compression:
 				Info->Entries[0].Buffer = realloc(Info->Entries[0].Buffer, Length + UnicodeLength(SMS->SMS[i].Text)*2 + 2);
 				if (Info->Entries[0].Buffer == NULL) return false;
 
 				memcpy(Info->Entries[0].Buffer+Length,SMS->SMS[i].Text,UnicodeLength(SMS->SMS[i].Text)*2);
 				Length=Length+UnicodeLength(SMS->SMS[i].Text)*2;
 				break;
+			default:
+				break;
 			}
 		}
 		Info->Entries[0].Buffer[Length]	  = 0;
@@ -939,6 +997,7 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 {
 	bool			*INPUTSorted, copyit;
 	int			i,OUTPUTNum,j,z,w;
+	GSM_SiemensOTASMSInfo	SiemensOTA,SiemensOTA2;
 
 	i = 0;
 	while (INPUT[i] != NULL) i++;
@@ -990,14 +1049,122 @@ GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage **INPUT, GSM_MultiSMSMessage **OUTPUT,
 			i++;
 			continue;
 		}
+		/* We have 1'st part of SIEMENS sms. It's single.
+		 * We will try to find other parts
+		 */
+		if (GSM_DecodeSiemensOTASMS(&SiemensOTA,&INPUT[i]->SMS[0]) &&
+		    SiemensOTA.PacketNum == 1) {
+			OUTPUT[OUTPUTNum] = malloc(sizeof(GSM_MultiSMSMessage));
+			if (OUTPUT[OUTPUTNum] == NULL) {
+				free(INPUTSorted);
+				return ERR_MOREMEMORY;
+			}
+			OUTPUT[OUTPUTNum+1] = NULL;
+
+			memcpy(&OUTPUT[OUTPUTNum]->SMS[0],&INPUT[i]->SMS[0],sizeof(GSM_SMSMessage));
+			OUTPUT[OUTPUTNum]->Number = 1;
+			INPUTSorted[i]	= true;
+			j		= 1;
+			/* We're searching for other parts in sequence */
+			while (j!=SiemensOTA.PacketsNum) {
+				z=0;
+				while(INPUT[z]!=NULL) {
+					/* This was sorted earlier or is not single */
+					if (INPUTSorted[z] || INPUT[z]->Number != 1) {
+						z++;
+						continue;
+					}
+					if (!GSM_DecodeSiemensOTASMS(&SiemensOTA2,&INPUT[z]->SMS[0])) {
+						z++;
+						continue;
+					}
+					if (SiemensOTA2.SequenceID != SiemensOTA.SequenceID ||
+					    SiemensOTA2.PacketNum != j+1 ||
+					    SiemensOTA2.PacketsNum != SiemensOTA.PacketsNum ||
+					    strcmp(SiemensOTA2.DataType,SiemensOTA.DataType) ||
+					    strcmp(SiemensOTA2.DataName,SiemensOTA.DataName)) {
+						z++;
+						continue;
+					}
+					/* For SMS_Deliver compare also SMSC and Sender number */
+					if (INPUT[z]->SMS[0].PDU == SMS_Deliver &&
+					    (strcmp(DecodeUnicodeString(INPUT[z]->SMS[0].SMSC.Number),DecodeUnicodeString(INPUT[i]->SMS[0].SMSC.Number)) ||
+					     strcmp(DecodeUnicodeString(INPUT[z]->SMS[0].Number),DecodeUnicodeString(INPUT[i]->SMS[0].Number)))) {
+						z++;
+						continue;
+					}
+					/* DCT4 Outbox: SMS Deliver. Empty number and SMSC. We compare dates */
+					if (INPUT[z]->SMS[0].PDU == SMS_Deliver 		&&
+					    UnicodeLength(INPUT[z]->SMS[0].SMSC.Number)==0 	&&
+					    UnicodeLength(INPUT[z]->SMS[0].Number)==0 		&&
+					    (INPUT[z]->SMS[0].DateTime.Day    != INPUT[i]->SMS[0].DateTime.Day 	  ||
+	   				     INPUT[z]->SMS[0].DateTime.Month  != INPUT[i]->SMS[0].DateTime.Month  ||
+					     INPUT[z]->SMS[0].DateTime.Year   != INPUT[i]->SMS[0].DateTime.Year   ||
+					     INPUT[z]->SMS[0].DateTime.Hour   != INPUT[i]->SMS[0].DateTime.Hour   ||
+					     INPUT[z]->SMS[0].DateTime.Minute != INPUT[i]->SMS[0].DateTime.Minute ||
+					     INPUT[z]->SMS[0].DateTime.Second != INPUT[i]->SMS[0].DateTime.Second)) {
+						z++;
+						continue;
+					}
+					dbgprintf("Found Siemens SMS %i\n",j);
+					/* We found correct sms. Copy it */
+					memcpy(&OUTPUT[OUTPUTNum]->SMS[j],&INPUT[z]->SMS[0],sizeof(GSM_SMSMessage));
+					OUTPUT[OUTPUTNum]->Number++;
+					INPUTSorted[z]=true;
+					break;
+				}
+				/* Incomplete sequence */
+				if (OUTPUT[OUTPUTNum]->Number==j) {
+					dbgprintf("Incomplete sequence\n");
+					break;
+				}
+				j++;
+			}
+			OUTPUTNum++;
+			i = 0;
+			continue;
+		}
+		/* We have some next Siemens sms from sequence */
+		if (GSM_DecodeSiemensOTASMS(&SiemensOTA,&INPUT[i]->SMS[0]) &&
+		    SiemensOTA.PacketNum > 1) {
+			j = 0;
+			while (INPUT[j]!=NULL) {
+				if (INPUTSorted[j]) {
+					j++;
+					continue;
+				}
+				/* We have some not unassigned first sms from sequence.
+				 * We can't touch other sms from sequences
+				 */
+				if (GSM_DecodeSiemensOTASMS(&SiemensOTA,&INPUT[j]->SMS[0]) &&
+				    SiemensOTA.PacketNum == 1) {
+					break;
+				}
+				j++;
+			}
+			if (INPUT[j]==NULL) {
+				OUTPUT[OUTPUTNum] = malloc(sizeof(GSM_MultiSMSMessage));
+				if (OUTPUT[OUTPUTNum] == NULL) {
+					free(INPUTSorted);
+					return ERR_MOREMEMORY;
+				}
+				OUTPUT[OUTPUTNum+1] = NULL;
+
+				memcpy(OUTPUT[OUTPUTNum],INPUT[i],sizeof(GSM_MultiSMSMessage));
+				INPUTSorted[i]=true;
+				OUTPUTNum++;
+				i = 0;
+				continue;
+			} else i++;
+		}
 		copyit = false;
 		/* If we have:
 		 * - linked sms returned by phone driver
 		 * - sms without linking
 		 * we copy it to OUTPUT
 		 */
-		if (INPUT[i]->Number 			!= 1 	       ||
-		    INPUT[i]->SMS[0].UDH.Type 		== UDH_NoUDH   ||
+		if (INPUT[i]->Number 			!= 1 	       	||
+		    INPUT[i]->SMS[0].UDH.Type 		== UDH_NoUDH   	||
                     INPUT[i]->SMS[0].UDH.PartNumber 	== -1) {
 			copyit = true;
 		}
